package com.xzzn.ems.service.impl;

import java.util.List;
import com.xzzn.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.xzzn.ems.mapper.EmsBatteryStackMapper;
import com.xzzn.ems.domain.EmsBatteryStack;
import com.xzzn.ems.service.IEmsBatteryStackService;

/**
 * 电池堆数据Service业务层处理
 *
 * @author xzzn
 * @date 2025-06-29
 */
@Service
public class EmsBatteryStackServiceImpl implements IEmsBatteryStackService
{
    @Autowired
    private EmsBatteryStackMapper emsBatteryStackMapper;

    /**
     * 查询电池堆数据
     *
     * @param id 电池堆数据主键
     * @return 电池堆数据
     */
    @Override
    public EmsBatteryStack selectEmsBatteryStackById(Long id)
    {
        return emsBatteryStackMapper.selectEmsBatteryStackById(id);
    }

    /**
     * 查询电池堆数据列表
     *
     * @param emsBatteryStack 电池堆数据
     * @return 电池堆数据
     */
    @Override
    public List<EmsBatteryStack> selectEmsBatteryStackList(EmsBatteryStack emsBatteryStack)
    {
        return emsBatteryStackMapper.selectEmsBatteryStackList(emsBatteryStack);
    }

    /**
     * 新增电池堆数据
     *
     * @param emsBatteryStack 电池堆数据
     * @return 结果
     */
    @Override
    public int insertEmsBatteryStack(EmsBatteryStack emsBatteryStack)
    {
        emsBatteryStack.setCreateTime(DateUtils.getNowDate());
        return emsBatteryStackMapper.insertEmsBatteryStack(emsBatteryStack);
    }

    /**
     * 修改电池堆数据
     *
     * @param emsBatteryStack 电池堆数据
     * @return 结果
     */
    @Override
    public int updateEmsBatteryStack(EmsBatteryStack emsBatteryStack)
    {
        emsBatteryStack.setUpdateTime(DateUtils.getNowDate());
        return emsBatteryStackMapper.updateEmsBatteryStack(emsBatteryStack);
    }

    /**
     * 批量删除电池堆数据
     *
     * @param ids 需要删除的电池堆数据主键
     * @return 结果
     */
    @Override
    public int deleteEmsBatteryStackByIds(Long[] ids)
    {
        return emsBatteryStackMapper.deleteEmsBatteryStackByIds(ids);
    }

    /**
     * 删除电池堆数据信息
     *
     * @param id 电池堆数据主键
     * @return 结果
     */
    @Override
    public int deleteEmsBatteryStackById(Long id)
    {
        return emsBatteryStackMapper.deleteEmsBatteryStackById(id);
    }
}
